#!/bin/bash

# The new way of doing this for Mac/Lin is with rsync, not rsyncd.
# rsyncd will be used only with Windows due to lack of native ssh.
# this allows most of the configuration and security to be kept on the
# server and only send the id_rsa.pub key and the client's rsync.password file
# The backup config is kept with the BackupPC server and uses passwordless ssh
# to connect to the client (via the server's public key -> ~/authorized_keys)

# It also removes the requirement to change /etc/rsyncd and have rsyncd running
# continuously.and/or messing with pre-existing rsyncd setups.
# the 'imalive.sh' script can be executed from the user dir, using the user cron.

# Given the emailed tarball from BackupPC of the files:
# Docs:
#  * SLA.txt - Service Level Agreement covering the backup
#  * Mac_Linux_Email_Instructions.[txt|html] instructions
#  * install_LinMac_backuppc_client.sh (this file)
#  * files2exclude (optional, copied from a single multiplatform version)
#  * Backup_Server_id_rsa.pub key >> ~/.ssh/authorized_keys
# (optional)
#    * imalive.sh (only for DHCP-supplied hosts)
#    * rsync.password - the client's rsync password for updating the client IP if
#      it's a DHCP client.

# so the only thing this needs to do is to:
# cp Backup_Server_id_rsa.pub key >> ~/.ssh/authorized_keys
# (optionally, if DHCP)
	#    insert the crontab entry to reference the imalive.sh
#    make sure the rsync.password file is chmod'ed correctly (and even this is
#    not a huge security hole as it only allows a write into the server's
#    /etc/BackupPC/alive dir (which is throttled as a very slow link,
#    and can't be read from
# the 'files2exclude' file just has to stay in the distro dir to be referenced by
# the rsync excutable.
#
# we could check the rsync executable and try to update it if it was too old, or
# other stuff, but we don't even have to deal with startup scripts.

# the installation on a Mac/Lin client is fairly straightforward..
# in fact we can now eliminate asking the user for UID/GID and handling that
# part.  The user can only backup the dirs she owns b/c of the shared key approach.
# The server can only connect to the account where it has the shared key.

#   TODO - Web form has to make this clear - the backup will read /EVERYTHING/
#   in the share/module that is not explicitly excluded by the 'files2exclude'
#   TODO how to update client rsync?
#      Windows, email with included URL to refresh the rsync + cygwin dll.
#      MacOSX - OS upgrades, or email with included URL to install a local rsync
#      Linux - like Mac

# - if client is a DHCP machine
#   - include the imalive script in the tarball
#   - add a line to /etc/crontab to have it execute every 30 min
# MacOSX
## min   hour    mday    month   wday    who     command
#36      14      *       *       *       *       /bin/sh /Users/hjm/testme.sh
# Linux (essentially same)
## m h dom mon dow user  command
#25,26,27,28,29,30 *    * * *   root    cd ${TESTDIR}/nacs/webbackuppc/alive && ./alive.py
# so we need only 1 crontab entry.

TEST=1 # this is a test if 1
#TESTDIR="$HOME/bupc_test"

RSYNC_RESPONSE=`rsync --version |grep version`
RSYNCBIN=`which rsync`
PWD=`pwd`
IAM=`whoami`
# next line assigns verbatim, except for the variable substitution - ie the variable
# gets '*' inserted.  However when echo'ing, unless the echo command is bracketed by
# set -f ; echo $CRONLINE; set +f
# it will glob filenames, so be careful.
# if we wanted /absolute/ verbatim, quote with ''
# the ${INSTALLDIR} var gets substituted at runtime, not immediately below
CRONLINE="*/30  *   *   *   *   /bin/sh ${HOME}/BackupPC_config/imalive.sh"
INSTALLDIR="${HOME}/BackupPC_config"
RSA_PUB="${INSTALLDIR}/BackupPC_Server_id_rsa.pub"
CRONTAB=""
INITSCRIPTDIR=""
INIT_SCR=""

pause() {
    echo "======================================"
    echo $1
    echo "Hit 'Enter' to continue."
    echo "======================================"
    read input
}
# set up the Mac/Linux- specific vars
OS=`uname` # Darwin or Linux
if [[ ${OS} =~ "Darwin" ]]; then
    OS="MacOSX" # switch to a more identifiable name
fi
cat << startinstall

*****************************************************************
    Starting the BackupPC client install using rsync method.
               Platform seems to be [${OS}]
*****************************************************************

startinstall

# if [ $OS == "Linux" ]; then
# 	if [[ ${TEST} = 1 ]]; then
# 		TESTDIR="/home/hjm" # assign to "" when done testing to delete the prefix
# 		pause "TESTDIR = ${TESTDIR}"
# 	fi
# else # for Mac
# 	if [[ ${TEST} = 1 ]]; then
# 		TESTDIR="/Users/hjm" # assign to "" when done testing to delete the prefix
# 	fi
# fi


if [ -d "$HOME/BackupPC_config" ]; then
	cd $HOME/BackupPC_config
else
	echo "ERROR: There's no [${HOME}/BackupPC_config] dir - did you move/rename the zipfile directory??"
	echo "Please see what's wrong and re-run this installation script."
	exit
fi
echo "chmod'ing rsync.password to keep it **secret**"
echo ""
chmod go-rx ${HOME}/BackupPC_config/rsync.password # keep the sectrets file secret

# Now need to append the backuppc@server's id_rsa.pub to user@client:~/.ssh/authorized_keys
# for the purpose of this script it will be called "BackupPC_Server_id_rsa.pub"
# Should be identical for Mac/Linux
if [ -e ${RSA_PUB} ]; then
	# append it onto the end of the 'authorized_keys' file.
	# if it gets appended 2x, is there any downside?  Or is it idemnepotent?
	if [ $TEST ]; then
		pause "About to copy in BackupPC server ssh key"
	fi
	cat $RSA_PUB >> ${HOME}/.ssh/authorized_keys
else
	echo "ERROR: BackupPC Server's ssh public key not where expected:"
	echo "        [${RSA_PUB}]"
	echo "Please see what's wrong and re-run this installation script."
	exit
fi


# if imalive.sh was included
if [[ -e ${INSTALLDIR}/imalive.sh ]]; then
#    echo "Copying [imalive.sh] reporter script to [${INSTALLDIR}].."
#    cp ${HOME/BackupPC_config}/imalive.sh  ${INSTALLDIR}   # then copy it into place
    # testing  for pre-existing ${CRONLINE} in ${CRONTAB}
	if [ $TEST ]; then
		pause "About to add crontab line.."
	fi
    EXISTING_CRONLINE=`crontab -l 2> /dev/null | grep imalive.sh`
    if [[ ! ${EXISTING_CRONLINE} =~ "imalive.sh" ]]; then
        # only insert if it's not already there
        echo "Adding cron line to CRONTAB for $USER"
        echo ""
        set -f # turn off globbing for next section
        crontab -l 2> /dev/null > ${HOME}/${USER}.crontab
        # now append the crontab, setting and unsetting the shell globbing
        echo "${CRONLINE}" >> ${HOME}/${USER}.crontab
        crontab -r 2> /dev/null # delete the current crontab
        # and read in the modified one.
        crontab ${HOME}/${USER}.crontab
        set +f # turn on globbing again
    else
    	echo ""
        echo "INFO: CRONTAB entry is already in ${CRONTAB} "
        echo "INFO: Assuming it's the result of previous installation attempt."
        echo ""
    fi
fi

pause "Checking your version of rsync."

if [[ ${RSYNC_RESPONSE} =~ "version" ]]; then
    # we have an rsync version
    # check for version 2.x
    if [[ ${RSYNC_RESPONSE} =~ '2.' && ${OS} = "MacOSX" ]]; then # it's a 2.x version
        echo << rsync2warning

WARNING: You appear to be running MacOSX and have a 2.x version of rsync.
You should really be running a 3.x version of rsync to retain all the
Mac-specific data in your files.  Please consider upgrading after this.

rsync2warning

# TODO: Provide a wgettable fat binary of rsync 3.x for Mac??

    fi
    # check for Version 3.x
    if [[ ${RSYNC_RESPONSE} =~ "3." ]]; then
         echo "EXCELLENT! You have a 3.x version of rsync."
    fi
fi
#########  The remaining Mac/Lin-specific stanzas are NOT NEEDED for
#########  this **rsync** version

# Final Goodbye.
cat <<goodbye

*****************************************************************
Everything has been set up for your backup.  Confirm the system
has been set up correctly, by going to:
   <http://dabrick.nacs.uci.edu/cgi-bin/BackupPC_Admin>
& use the login and password you set when you filled in the form.
If you've forgotten, the password is in plain text in the file
    /where/you/unpacked/the/email/attachment/rsync.secrets
(which should be set to be readable ONLY BY YOU).
Then select your PC/Mac from the dropdown menu  and try to
initiate a backup.
Let me know if you have problems.
Harry Mangalam <harry.mangalam@uci.edu>
*****************************************************************

goodbye
